-- A3Pv2.6/VectGen/VectGen.vhdl
-- created dim. déc.  8 08:36:48 CET 2019 by whygee@f-cpu.org
-- version dim. déc.  8 22:01:42 CET 2019 : gets the DUT's vector size.
--
-- Released under the GNU AGPLv3 license

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
use STD.textio.all;
Library SLV;
    use SLV.SLV_utils.all;
Library work;
    use work.all;
Library proasic3;
    use proasic3.all;
    use proasic3.PA3_genlib.all;

entity VectGen is
  generic (
    VectGenWidthIn : integer := 200;
    VectGenWidthOut: integer := 100;
    filename : string := ""; -- file containing the exclude vectors
    verbose : string := ""  
  );
end VectGen;

architecture plip of VectGen is
  signal VectIn : std_logic_vector(VectGenWidthIn -1 downto 0) := (others=>'0');
  signal VectOut: std_logic_vector(VectGenWidthOut-1 downto 0) := (others=>'0');
  signal VectClk: std_logic := '0';

  shared variable VectInMax  : integer := VectGenWidthIn -1;
  shared variable VectOutMax : integer := VectGenWidthOut-2; -- -1 is used by VectInMax loopback

begin

  -- force the registration of the gates when placed at the very beginning
  rg: entity register_generics generic map(
         gate_select_number => 1, -- no listing
         bit_select_number => -1, -- no alteration
         flip_LUT => 0,           -- trace mode
         filename => filename,    -- eventual exclusions
         verbose => verbose)
  port map(clock => '0');   -- the updates are directly driven by main

  -- This code is executed after all the variables/signals are initialised
  main: process

    procedure GetVectorsSize is
    begin
      wait for 1 ms;  -- let the circuit settle so we can read the output

      -- Find the size of the output vector
      FindOut: loop
        assert VectOutMax > 1
             report " Can't find the output vector size :-( "
             severity failure;
        exit when VectOut(VectOutMax) = '1';
        VectOutMax := VectOutMax - 1;
      end loop;

      -- Then find the size of the input vector
      FindIn: loop
        assert VectInMax > 1
          report " Can't find the intput vector size :-( "
          severity failure;
        VectIn(VectInMax) <= '1';
        wait for 1 ns;
        exit when VectOut(VectGenWidthOut-1) = '1';
        VectInMax := VectInMax - 1;
      end loop;

      if verbose /= "" then
        report " Input vector : " & integer'image(VectInMax)
            & " bits, Output vector : " & integer'image(VectOutMax) & " bits";
      end if;
    end GetVectorsSize;


  begin
    GetVectorsSize;

--    update_histogram;
--    display_histogram;
    wait;
  end process;

  -- Here we "plug" the wrapper to the Vector Generator ports.
  -- It must be the last section of the code so the drivers
  -- can initialise all the data behind the scene.
  wrap: entity VG_Wrapper
    generic map(
      VectGenWidthIn  => VectGenWidthIn,
      VectGenWidthOut => VectGenWidthOut)
    port map (
      VectIn  => VectIn,
      VectOut => VectOut,
      VectClk => VectClk);

end plip;



